package com.example.meshtest

import android.Manifest
import android.bluetooth.BluetoothAdapter
import android.bluetooth.BluetoothManager
import android.bluetooth.le.BluetoothLeScanner
import android.bluetooth.le.ScanCallback
import android.bluetooth.le.ScanFilter
import android.bluetooth.le.ScanResult
import android.bluetooth.le.ScanSettings
import android.content.pm.PackageManager
import android.os.Build
import android.os.Bundle
import android.os.Handler
import android.os.ParcelUuid
import android.util.Log
import android.view.View
import android.widget.Toast
import androidx.annotation.RequiresApi
import androidx.annotation.RequiresPermission
import androidx.appcompat.app.AppCompatActivity
import androidx.core.app.ActivityCompat
import androidx.core.content.ContextCompat
import com.example.meshtest.databinding.ActivityMainBinding
import no.nordicsemi.android.mesh.MeshManagerApi
import java.util.ArrayList
import java.util.UUID


class MainActivity : AppCompatActivity() {
    private lateinit var binding: ActivityMainBinding
    private lateinit var mFilterUuid : UUID
    private var mMeshManager: Mesh? = null
    private var mMeshManagerApi: MeshManagerApi? = null
    private var bluetoothLeScanner :BluetoothLeScanner?=null
    private var scanning = false
    private val handler = Handler()
    private var bluetoothAdapter: BluetoothAdapter? = null
    val LOCATION_PERMISSION_REQUEST_CODE: Int = 1
    val BLUETOOTH_PERMISSION_REQUEST_CODE: Int = 2
    var temp = false

    @RequiresPermission(Manifest.permission.BLUETOOTH_SCAN)
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        // Inflate the layout using View Binding
        binding = ActivityMainBinding.inflate(layoutInflater)
        setContentView(binding.root)
        binding.connected.text="Connecting"
        binding.identify.visibility = View.GONE
        // Access views using binding
        binding.identify.setOnClickListener {
            if(!temp) {
                mMeshManager!!.identifyDevice()
                temp = true
                binding.identify.text="Provision"
            } else {
                mMeshManager!!.provisionDevice()
            }
        }

        checkPermissions();
    }

    @RequiresApi(Build.VERSION_CODES.S)
    private fun checkPermissions() {
        // Check for location permissions
        if (ContextCompat.checkSelfPermission(
                this,
                Manifest.permission.ACCESS_FINE_LOCATION
            ) != PackageManager.PERMISSION_GRANTED
        ) {
            ActivityCompat.requestPermissions(
                this, arrayOf<String>(
                    Manifest.permission.ACCESS_FINE_LOCATION,
                    Manifest.permission.ACCESS_COARSE_LOCATION,
                    Manifest.permission.BLUETOOTH_SCAN,
                    Manifest.permission.BLUETOOTH_CONNECT
                ), LOCATION_PERMISSION_REQUEST_CODE
            )
        } else {
            // Permissions are already granted, proceed with Bluetooth operations
            proceedWithBluetoothOperations()
        }
    }

    override fun onRequestPermissionsResult(
        requestCode: Int,
        permissions: Array<String?>,
        grantResults: IntArray
    ) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults)
        if (requestCode == LOCATION_PERMISSION_REQUEST_CODE) {
            if (grantResults.size > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                // Permissions granted, proceed with Bluetooth operations
                proceedWithBluetoothOperations()
            } else {
                // Permissions denied
                Toast.makeText(
                    this,
                    "Location permissions are required for Bluetooth scanning.",
                    Toast.LENGTH_SHORT
                ).show()
            }
        }
    }

    private fun proceedWithBluetoothOperations() {
        val manager = getSystemService(BLUETOOTH_SERVICE) as BluetoothManager
        bluetoothAdapter = manager.adapter
        bluetoothLeScanner = bluetoothAdapter!!.bluetoothLeScanner
        mMeshManager = Mesh(this)
        // Your code to perform Bluetooth operations goes here
        scanLeDevice()
    }

    // Stops scanning after 10 seconds.
    private val SCAN_PERIOD: Long = 10000

    @RequiresPermission(Manifest.permission.BLUETOOTH_SCAN)
    private fun scanLeDevice() {
        //Scanning settings
        val settings = ScanSettings.Builder()
            .setScanMode(ScanSettings.SCAN_MODE_LOW_LATENCY) // Refresh the devices list every second
            .setReportDelay(0) // Hardware filtering has some issues on selected devices
            /*.setUseHardwareBatchingIfSupported(false)*/
            .build()
        mFilterUuid=Mesh.MESH_PROVISIONING_UUID

        //Let's use the filter to scan only for unprovisioned mesh nodes.
        val filters: MutableList<ScanFilter> = ArrayList()
        filters.add(ScanFilter.Builder().setServiceUuid(ParcelUuid((Mesh.MESH_PROVISIONING_UUID))).build())
        if (!scanning) { // Stops scanning after a pre-defined scan period.
            scanning = true
            bluetoothLeScanner!!.startScan(filters, settings, mScanCallbacks)
        } else {
            scanning = false
            bluetoothLeScanner!!.stopScan(mScanCallbacks)
        }
    }

    private val mScanCallbacks: ScanCallback = object : ScanCallback() {
        @RequiresPermission(Manifest.permission.BLUETOOTH_SCAN)
        override fun onScanResult(callbackType: Int, result: ScanResult) {
            try {
                if (mFilterUuid == Mesh.MESH_PROVISIONING_UUID) {
                    updateScannerLiveData(result)
                } else if (mFilterUuid == Mesh.MESH_PROXY_UUID) {
                    val serviceData: ByteArray? =
                        getServiceData(result, Mesh.MESH_PROXY_UUID)
                    if (mMeshManagerApi != null) {
                        if (mMeshManagerApi!!.isAdvertisingWithNetworkIdentity(serviceData)) {
                            if (mMeshManagerApi!!.networkIdMatches(serviceData)) {
                                updateScannerLiveData(result)
                            }
                        } else if (mMeshManagerApi!!.isAdvertisedWithNodeIdentity(serviceData)) {
                            if (serviceData?.let { checkIfNodeIdentityMatches(it) } == true) {
                                updateScannerLiveData(result)
                            }
                        }
                    }
                }
            } catch (ex: Exception) {
                Log.e(
                    "MeshFragment",
                    "Error: " + ex.message
                )
            }
        }

        override fun onBatchScanResults(results: List<ScanResult>) {
        }

        override fun onScanFailed(errorCode: Int) {
            scanning=false
        }
    }

    private fun checkIfNodeIdentityMatches(serviceData: ByteArray): Boolean {
        val network = mMeshManagerApi!!.meshNetwork
        if (network != null) {
            for (node in network.nodes) {
                if (mMeshManagerApi!!.nodeIdentityMatches(node, serviceData)) {
                    return true
                }
            }
        }
        return false
    }
    fun getServiceData(result: ScanResult, serviceUuid: UUID): ByteArray? {
        val scanRecord = result.scanRecord
        if (scanRecord != null) {
            return scanRecord.getServiceData(ParcelUuid((serviceUuid)))
        }
        return null
    }

    @RequiresPermission(Manifest.permission.BLUETOOTH_SCAN)
    private fun updateScannerLiveData(result: ScanResult) {
        val scanRecord = result.scanRecord
        Log.d("MeshManager", "found "+scanRecord!!.deviceName)
        if (scanRecord.bytes != null) {
            mMeshManager!!.connectToMeshDevice(result)
        }
    }
    fun onDeviceConnected(name: String) {
        binding.connected.text="Connected to "+name
        binding.identify.visibility = View.VISIBLE
    }
}